@@ -1,5 +1,6 @@
 """
-File for processing an input datafile into a PyTorch-compatible format.
+Module with functions for processing an input datafile into a PyTorch-compatible
+format
 
 .............................................................................
 idptools-parrot was developed by the Holehouse lab
@@ -20,7 +21,7 @@ from torch.utils.data import Dataset
 
 from parrot import encode_sequence
 
-def parse_file(tsvfile, datatype, problem_type, excludeSeqID=False):
+def parse_file(tsvfile, datatype, problem_type, num_classes, excludeSeqID=False):
 	"""Parse a datafile containing sequences and values.
 
 	Each line of of the input tsv file contains a sequence of amino acids, a value
@@ -64,26 +65,40 @@ def parse_file(tsvfile, datatype, problem_type, excludeSeqID=False):
 	with open(tsvfile) as f:
 		lines = [line.rstrip().split() for line in f]
 
-		# Add a dummy seqID if none are provided
-		if excludeSeqID:
-			for line in lines:
-				line.insert(0, '')
+	# Add a dummy seqID if none are provided
+	if excludeSeqID:
+		for line in lines:
+			line.insert(0, '')
 
+	try:
 		if datatype == 'residues':		# A value for each residue in a sequence
 			data = [[x[0], x[1], np.array( x[2:], dtype=np.float)] for x in lines]
 		elif datatype == 'sequence':	# A single value per sequence
 			data = [[x[0], x[1], float(x[2])] for x in lines]
 		else:
-			print('Invalid datatype.')
+			raise ValueError('Invalid datatype. Must be "residues" or "sequence".')
+	except:
+		raise Exception("Input data is not correctly formatted for datatype '%s'" % datatype)
 
 	if problem_type == 'classification':
 		if datatype == 'sequence':
 			for sample in data:
 				sample[2] = int(sample[2])
 
+				# Validate that all of the class labels are valid
+				if sample[2] >= num_classes or sample[2] < 0:
+					raise ValueError("Invalid class label: %s" % sample[0])
+
 		else:
 			for sample in data:
 				sample[2] = list(map(int, sample[2]))
+				test = np.array(sample[2])
+
+				if np.any(test < 0) or np.any(test >= num_classes):
+					raise ValueError("Invalid class label: %s" % sample[0])
+				elif len(test) != len(sample[1]):
+					raise Exception(
+					'Input not properly formatted. Number of values must be equal to sequence length: %s' % sample[0])
 	return data
 
 class SequenceDataset(Dataset):
@@ -100,8 +115,12 @@ class SequenceDataset(Dataset):
 		have the format: [seqID, sequence, value(s)]
 	encoding_scheme : str
 		Description of how an amino acid sequence should be encoded as a numeric 
-		vector. Providing a string other than 'onehot' or 'biophysics' will 
-		produce unintended consequences.
+		vector. Providing a string other than 'onehot', 'biophysics', or 'user' 
+		will produce unintended consequences.
+	encoder: UserEncoder object, optional
+		If encoding_scheme is 'user', encoder should be a UserEncoder object
+		that can convert amino acid sequences to numeric vectors. If
+		encoding_scheme is not 'user', use None.
 	"""
 
 	def __init__(self, data, subset=np.array([]), encoding_scheme='onehot', 
@@ -116,12 +135,14 @@ class SequenceDataset(Dataset):
 			Array containing the indices of `data` that are to be a part of this
 			dataset. Providing an empty array causes all of `data` to be integrated
 			into the dataset (default is empty array).
-		encoding_scheme : str, optional
+		encoding_scheme : str
 			Description of how an amino acid sequence should be encoded as a numeric 
-			vector. Providing a string other than 'onehot' or 'biophysics' will 
-			produce unintended consequences (default is 'onehot').
-		encoder : UserEncoder object, optional
-			TODO: write me
+			vector. Providing a string other than 'onehot', 'biophysics', or 'user' 
+			will produce unintended consequences.
+		encoder: UserEncoder object, optional
+			If encoding_scheme is 'user', encoder should be a UserEncoder object
+			that can convert amino acid sequences to numeric vectors. If
+			encoding_scheme is not 'user', use None.
 		"""
 
 		self.encoding_scheme = encoding_scheme
@@ -151,7 +172,7 @@ class SequenceDataset(Dataset):
 		Returns
 		-------
 		tuple
-			a tuple of a sequence vector and its corresponding values
+			a tuple of a name, sequence vector and its corresponding values
 		"""
 
 		if torch.is_tensor(idx):
@@ -164,8 +185,9 @@ class SequenceDataset(Dataset):
 		elif self.encoding_scheme == 'user':
 			sequence_vector = self.encoder.encode(self.data[idx][1])
 
+		name = self.data[idx][0]
 		value = self.data[idx][2]
-		sample = (sequence_vector, value)
+		sample = (name, sequence_vector, value)
 
 		return sample
 
@@ -185,11 +207,12 @@ def seq_class_collate(batch):
 	Returns
 	-------
 	tuple
-		a tuple with concatenated sequence_vectors and target_values(s)
+		a tuple with concatenated names, sequence_vectors and target_values
 	"""
 
-	orig_seq_vectors = [item[0] for item in batch]
-	orig_targets = [item[1] for item in batch]
+	names = [item[0] for item in batch]
+	orig_seq_vectors = [item[1] for item in batch]
+	orig_targets = [item[2] for item in batch]
 
 	longest_seq = len(max(orig_seq_vectors, key=lambda x: len(x)))
 
@@ -198,10 +221,10 @@ def seq_class_collate(batch):
 	for i,j in enumerate(orig_seq_vectors):
 		padded_seq_vectors[i][0:len(j)] = j
 
-	padded_seq_vectors = torch.FloatTensor(padded_seq_vectors)  #CHANGEME
+	padded_seq_vectors = torch.FloatTensor(padded_seq_vectors)
 	targets = torch.LongTensor(orig_targets)
 
-	return (padded_seq_vectors, targets)
+	return (names, padded_seq_vectors, targets)
 
 
 def seq_regress_collate(batch):
@@ -219,11 +242,12 @@ def seq_regress_collate(batch):
 	Returns
 	-------
 	tuple
-		a tuple with concatenated sequence_vectors and target_values(s)
+		a tuple with concatenated names, sequence_vectors and target_value
 	"""
 
-	orig_seq_vectors = [item[0] for item in batch]
-	orig_targets = [[item[1]]for item in batch]
+	names = [item[0] for item in batch]
+	orig_seq_vectors = [item[1] for item in batch]
+	orig_targets = [[item[2]]for item in batch]
 
 	longest_seq = len(max(orig_seq_vectors, key=lambda x: len(x)))
 
@@ -235,7 +259,7 @@ def seq_regress_collate(batch):
 	padded_seq_vectors = torch.FloatTensor(padded_seq_vectors)
 	targets = torch.FloatTensor(orig_targets)
 
-	return (padded_seq_vectors, targets)
+	return (names, padded_seq_vectors, targets)
 
 def res_class_collate(batch):
 	"""Collates sequences and their values into a batch
@@ -254,11 +278,12 @@ def res_class_collate(batch):
 	Returns
 	-------
 	tuple
-		a tuple with concatenated sequence_vectors and target_values(s)
+		a tuple with concatenated names, sequence_vectors and target_values
 	"""
 
-	orig_seq_vectors = [item[0] for item in batch]
-	orig_targets = [item[1] for item in batch]
+	names = [item[0] for item in batch]
+	orig_seq_vectors = [item[1] for item in batch]
+	orig_targets = [item[2] for item in batch]
 
 	longest_seq = len(max(orig_seq_vectors, key=lambda x: len(x)))
 
@@ -273,7 +298,8 @@ def res_class_collate(batch):
 
 	padded_seq_vectors = torch.FloatTensor(padded_seq_vectors)
 	padded_targets = torch.LongTensor(padded_targets)
-	return (padded_seq_vectors, padded_targets)
+
+	return (names, padded_seq_vectors, padded_targets)
 
 def res_regress_collate(batch):
 	"""Collates sequences and their values into a batch
@@ -292,11 +318,12 @@ def res_regress_collate(batch):
 	Returns
 	-------
 	tuple
-		a tuple with concatenated sequence_vectors and target_values(s)
+		a tuple with concatenated names, sequence_vectors and target_values
 	"""
 
-	orig_seq_vectors = [item[0] for item in batch]
-	orig_targets = [item[1] for item in batch]
+	names = [item[0] for item in batch]
+	orig_seq_vectors = [item[1] for item in batch]
+	orig_targets = [item[2] for item in batch]
 
 	longest_seq = len(max(orig_seq_vectors, key=lambda x: len(x)))
 
@@ -313,7 +340,7 @@ def res_regress_collate(batch):
 	padded_targets = torch.FloatTensor(padded_targets).view(
 			(len(padded_targets), len(padded_targets[0]), 1))
 
-	return (padded_seq_vectors, padded_targets)
+	return (names, padded_seq_vectors, padded_targets)
 
 
 def vector_split(v, fraction):
@@ -368,9 +395,9 @@ def read_split_file(split_file):
 		test_samples = np.array([int(i) for i in lines[2]])
 	return training_samples, val_samples, test_samples
 
-def split_data(data_file, datatype, problem_type, excludeSeqID=False, 
-			split_file=None, encoding_scheme='onehot', encoder=None,
-			percent_val=0.15, percent_test=0.15):
+def split_data(data_file, datatype, problem_type, num_classes, 
+			excludeSeqID=False, split_file=None, encoding_scheme='onehot', 
+			encoder=None, percent_val=0.15, percent_test=0.15):
 	"""Divide a datafile into training, validation, and test datasets
 
 	Takes in a datafile and specification of the data format and the machine
@@ -406,9 +433,10 @@ def split_data(data_file, datatype, problem_type, excludeSeqID=False,
 	encoding_scheme : str, optional
 		The method to be used for encoding protein sequences as numeric vectors.
 		Currently 'onehot' and 'biophysics' are implemented (default is 'onehot').
-	encoder : UserEncoder object, optional
-		TODO: write me
-
+	encoder: UserEncoder object, optional
+		If encoding_scheme is 'user', encoder should be a UserEncoder object
+		that can convert amino acid sequences to numeric vectors. If
+		encoding_scheme is not 'user', use None.
 	percent_val : float, optional
 		If `split_file` is not provided, the fraction of the data that should be
 		randomly assigned to the validation set. Should be in the range [0-1]
@@ -430,7 +458,7 @@ def split_data(data_file, datatype, problem_type, excludeSeqID=False,
 		a dataset containing the test set sequences and values
 	"""
 
-	data = parse_file(data_file, datatype, problem_type, excludeSeqID=excludeSeqID)
+	data = parse_file(data_file, datatype, problem_type, num_classes, excludeSeqID=excludeSeqID)
 	num_samples = len(data)
 
 	if split_file == None:
@@ -464,7 +492,7 @@ def split_data(data_file, datatype, problem_type, excludeSeqID=False,
 
 	return train_set, val_set, test_set
 
-def split_data_cv(data_file, datatype, problem_type, excludeSeqID=False, 
+def split_data_cv(data_file, datatype, problem_type, num_classes, excludeSeqID=False, 
 					split_file=None, encoding_scheme='onehot', encoder=None,
 					percent_val=0.15, percent_test=0.15, n_folds=5):
 	"""Divide a datafile into training, val, test and 5 cross-val datasets.
@@ -503,9 +531,10 @@ def split_data_cv(data_file, datatype, problem_type, excludeSeqID=False,
 	encoding_scheme : str, optional
 		The method to be used for encoding protein sequences as numeric vectors.
 		Currently 'onehot' and 'biophysics' are implemented (default is 'onehot').
-	encoder : UserEncoder object, optional
-		TODO: write me
-
+	encoder: UserEncoder object, optional
+		If encoding_scheme is 'user', encoder should be a UserEncoder object
+		that can convert amino acid sequences to numeric vectors. If
+		encoding_scheme is not 'user', use None.
 	percent_val : float, optional
 		If `split_file` is not provided, the fraction of the data that should be
 		randomly assigned to the validation set. Should be in the range [0-1]
@@ -532,7 +561,7 @@ def split_data_cv(data_file, datatype, problem_type, excludeSeqID=False,
 		a dataset containing the test set sequences and values
 	"""
 
-	data = parse_file(data_file, datatype, problem_type, excludeSeqID=excludeSeqID)
+	data = parse_file(data_file, datatype, problem_type, num_classes, excludeSeqID=excludeSeqID)
 	n_samples = len(data)
 
 	# Initial step: split into training, val, and test sets
